#include "keyboard.h"
#include "os.h"

int ShiftKeyDown; // Variable for Shift Key Down

/* Wait until buffer is empty */
void keyboard_init()
{
    while (inportb(0x64)&1)
      inportb(0x60);
};

unsigned int FetchAndAnalyzeScancode()
{
	unsigned int scancode; // For getting the keyboard scancode
	while(TRUE) // Loop until a key (w/o shift key) has been pressed
	{
		scancode = inportb(0x60);   // 0x60: get scan code from the keyboard

		if ( scancode & 0x80 ) // Key released? Check bit 7 (10000000b = 0x80) of scan code for this
		{
			scancode &= 0x7F; // Key was released, compare only low seven bits: 01111111b = 0x7F
			if ( scancode == KRLEFT_SHIFT || scancode == KRRIGHT_SHIFT ) // A key was released, shift key up?
			{
			    ShiftKeyDown = 0;	// yes, it is up --> NonShift
			}
		}
		else // Key was pressed.
		{
		    // Capture scan code of shift key, if pressed
		    if ( scancode == KRLEFT_SHIFT || scancode == KRRIGHT_SHIFT )
		    {
			    ShiftKeyDown = 1; // It is down, use asciiShift characters
			    continue; // Loop, so it will not return a scan code for the shift key
		    }
		}
	    break; // Leave the loop
	}
	return scancode;
}

unsigned char const k_getch() // Scancode --> ASCII
{
	unsigned int scan;		  // scancode from the keyboard
	unsigned char retchar;	  // The chararacter that returns the scan code to ASCII code
	scan = FetchAndAnalyzeScancode(); // Grab scancode, and get the position of the shift key
	if ( ShiftKeyDown )
	    retchar = asciiShift[scan];	// (Upper) Shift Codes
	else
		retchar = asciiNonShift[ scan ]; // (Lower) Non-Shift Codes
	return retchar; // ASCII version
}

void keyboard_handler(struct regs* r)
{
   k_printf("keyboard handler works!", 8, 0x2);

   unsigned char bufferKEY[10];
   bufferKEY[0] = k_getch();
   k_printf(bufferKEY, 10,0xA); // the ASCII character
}

void keyboard_install()
{
    /* Installs 'keyboard_handler' to IRQ1 */
    irq_install_handler(1, keyboard_handler);
    keyboard_init();
}



